/*
 * Pad for Psemu Pro like Emulators
 *
 * By ammoQ <ammoq@ammoq.com>
 * Based on padXwin by linuzappz <linuzappz@hotmail.com>
 *
 */


/* Definition for AXIS; just a wild guess...*/
/* to enable a second pad, change -1 to the correct value */
int AxDigitalX[2] = { 0, -1 };
int AxDigitalY[2] = { 1, -1 };
int AxLeftX[2] = { 2, -1 };
int AxLeftY[2] = { 3, -1 };
int AxRightX[2] = { 4, -1 };
int AxRightY[2] = { 5, -1 };

#define PCSX_STYLE 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/joystick.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>

typedef char HWND;

char *LibName = "ammoQ Analog Pad Driver";

const unsigned char version = 1; // PSEmu 1.x library
const unsigned char revision = VERSION;
const unsigned char build = BUILD;

// Filenames for device files, e.g. "/dev/input/js0"
char devicefilename[2][100] = {"/dev/input/js0", ""};

// File desciptors for device files
int devicefile[2] = { -1, -1 };

// Position of directional buttons in PadButtons
// Never change this!
#define PSXBTN_UP 12
#define PSXBTN_RIGHT 13
#define PSXBTN_DOWN 14
#define PSXBTN_LEFT 15

// Dead range (for analogue sticks)
#define MINZERO -1500
#define MAXZERO 1500

// Assignment of PSX buttons to device buttons
short PadButtons[2][16] = {{
    10,	// L2
    11, // R2
    6,  // L1
    7,	// R1
    4,	// Triangle
    1,	// Circle
    0,	// Cross
    3,	// Square
    9,  // Select
    12,	// Left Analog???
    13,	// Right Analog???
     8, // Start
    -1,	// Up
    -1,	// Right
    -1,	// Down
    -1	// Left
},
{ -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1 }};

int PadOpened = 0;
int PadFlags = 0;

char *PSEgetLibName(void) {
    return LibName;
}

unsigned long PSEgetLibType(void) {
    return 8; // PSE_LT_PAD
}

unsigned long PSEgetLibVersion(void) {
    return version<<16|revision<<8|build;
}

long PADinit(long flags) {
    PadFlags|=flags;

    return 0;
}

long PADshutdown(void) {
    return 0;
}

long PADopen(unsigned long *Disp) {
    int i;
    PadOpened = 1;

    for (i=0; i<2; i++) {
        if (i==1 && !strcmp(devicefilename[i], devicefilename[0])) {
            devicefile[i] = devicefile[0];
        }
        else
        if (devicefilename[i][0]) {
            devicefile[i] = open(devicefilename[i], O_RDONLY);
            if (devicefile[i] == -1) {
                fprintf(stderr, "padAnalog: could not open %s\n", devicefilename[i]);
            }
        }
    }

    return 0;
}

long PADclose(void) {
    int i;
    for (i=0; i<2; i++) {
        if (i==1 && !strcmp(devicefilename[i], devicefilename[0])) {
	    /* nothing to do */
	}
	else
        if (devicefile[i] > -1) {
            close (devicefile[i]);
        }
    }

    PadOpened = 0;
    return 0;
}

long PADquery(void) {
    return 3; // both pads
}

typedef struct
{
    unsigned char controllerType;
    unsigned short buttonStatus;
    unsigned char rightJoyX, rightJoyY, leftJoyX, leftJoyY;
    unsigned char moveX, moveY;
    unsigned char reserved[91];
} PadDataS;


short PressedKey;
short ReleasedKey;
unsigned short PadStat[2] = {0xffff, 0xffff};

int leftX[2] = {127,127};
int leftY[2] = {127,127};
int rightX[2] = {127,127};
int rightY[2] = {127,127};

void CheckPad(int pad) {
    fd_set rfds;
    struct timeval tv;
    int retval;
    struct js_event e;
    int i;

    if (pad == 1 && devicefile[0] == devicefile[1]) {
        // pad 1 has already been checked
        return;
    }

    if (devicefile[pad] >-1) {
        do {
            // Test if there is input waiting for us...
            FD_ZERO(&rfds);
            FD_SET(devicefile[pad], &rfds);
            tv.tv_sec = 0;
            tv.tv_usec = 0;

            retval = select(devicefile[pad]+1, &rfds, NULL, NULL, &tv);
            if (retval) {
                read (devicefile[pad], &e, 8);

                if (e.type == JS_EVENT_AXIS) {
                    // this is the stick
                    if (e.number == AxDigitalX[pad]) {
                        // left-right axis
                        if (e.value > MAXZERO) {
                            // right
                            PadStat[pad]&=~(1<<PSXBTN_RIGHT);
                            PadStat[pad]|=(1<<PSXBTN_LEFT); 
                        }
                        else if (e.value < MINZERO) {
                            // left
                            PadStat[pad]&=~(1<<PSXBTN_LEFT);
                            PadStat[pad]|=(1<<PSXBTN_RIGHT); 
                        }
                        else {
                            PadStat[pad]|=(1<<PSXBTN_LEFT); 
                            PadStat[pad]|=(1<<PSXBTN_RIGHT); 
                        }
                    }
                    else if (e.number == AxDigitalY[pad]) {
                        // up-down axis
                        if (e.value > MAXZERO) {
                            // down
                            PadStat[pad]&=~(1<<PSXBTN_DOWN);
                            PadStat[pad]|=(1<<PSXBTN_UP); 
                        }
                        else if (e.value < MINZERO) {
                            // up
                            PadStat[pad]&=~(1<<PSXBTN_UP);
                            PadStat[pad]|=(1<<PSXBTN_DOWN); 
                        }
                        else {
                             PadStat[pad]|=(1<<PSXBTN_UP); 
                             PadStat[pad]|=(1<<PSXBTN_DOWN); 
                        }
                    }
                    else if (e.number == AxLeftX[pad]) {
                      leftX[pad] = (e.value+32768)>>8;
                    }
                    else if (e.number == AxLeftY[pad]) {
                      leftY[pad] = (e.value+32768)>>8;
                    }
                    else if (e.number == AxRightX[pad]) {
                      rightX[pad] = (e.value+32768)>>8;
                    }
                    else if (e.number == AxRightY[pad]) {
                      rightY[pad] = (e.value+32768)>>8;
                    }
                }
                else if (e.type == JS_EVENT_BUTTON) {
                    for (i=0; i<16; i++) {
                        if (PadButtons[pad][i] == e.number) {
                            if (e.value)
                                PadStat[pad]&=~(1<<i);
                            else
                                PadStat[pad]|=(1<<i); 
                        }
                    }
                }

		if (pad == 0 && devicefile[0] == devicefile[1]) {
                    // check the other pad, too
                    if (e.type == JS_EVENT_AXIS) {
                        // this is the stick
                        if (e.number == AxDigitalX[1]) {
                            // left-right axis
                            if (e.value > MAXZERO) {
                                // right
                                PadStat[1]&=~(1<<PSXBTN_RIGHT);
                                PadStat[1]|=(1<<PSXBTN_LEFT); 
                            }
                            else if (e.value < MINZERO) {
                                // left
                                PadStat[1]&=~(1<<PSXBTN_LEFT);
                                PadStat[1]|=(1<<PSXBTN_RIGHT); 
                            }
                            else {
                                PadStat[1]|=(1<<PSXBTN_LEFT); 
                                PadStat[1]|=(1<<PSXBTN_RIGHT); 
                            }
                        }
                        else if (e.number == AxDigitalY[1]) {
                            // up-down axis
                            if (e.value > MAXZERO) {
                                // down
                                PadStat[1]&=~(1<<PSXBTN_DOWN);
                                PadStat[1]|=(1<<PSXBTN_UP); 
                            }
                            else if (e.value < MINZERO) {
                                // up
                                PadStat[1]&=~(1<<PSXBTN_UP);
                                PadStat[1]|=(1<<PSXBTN_DOWN); 
                            }
                            else {
                                 PadStat[1]|=(1<<PSXBTN_UP); 
                                 PadStat[1]|=(1<<PSXBTN_DOWN); 
                            }
                        }
                        else if (e.number == AxLeftX[1]) {
                          leftX[1] = (e.value+32768)>>8;
                        }
                        else if (e.number == AxLeftY[1]) {
                          leftY[1] = (e.value+32768)>>8;
                        }
                        else if (e.number == AxRightX[1]) {
                          rightX[1] = (e.value+32768)>>8;
                        }
                        else if (e.number == AxRightY[1]) {
                          rightY[1] = (e.value+32768)>>8;
                        }
                    }
                    else if (e.type == JS_EVENT_BUTTON) {
                        for (i=0; i<16; i++) {
                            if (PadButtons[1][i] == e.number) {
                                if (e.value)
                                    PadStat[1]&=~(1<<i);
                                else
                                    PadStat[1]|=(1<<i); 
                            }
                        }
                    }
		}
            } // if retval
        } while (retval);
    }  // if devicefile[pad] open
}
    

long PADreadPort1(PadDataS *pad) {
    CheckPad(0);

    pad->buttonStatus = PadStat[0];
    pad->controllerType = 7; // analog Pad
    pad->rightJoyX = rightX[0];
    pad->rightJoyY = rightY[0];
    pad->leftJoyX = leftX[0];
    pad->leftJoyY = leftY[0];

    if (PCSX_STYLE) { // ePSXe different from pcsx, swap bytes
       pad->buttonStatus = (pad->buttonStatus>>8)|(pad->buttonStatus<<8);
    }

    return 0;
}

long PADreadPort2(PadDataS *pad) {
    CheckPad(1);

    pad->buttonStatus = PadStat[1];
    pad->controllerType = 7; // analog Pad
    pad->rightJoyX = rightX[1];
    pad->rightJoyY = rightY[1];
    pad->leftJoyX = leftX[1];
    pad->leftJoyY = leftY[1];

    if (PCSX_STYLE) { // ePSXe different from pcsx, swap bytes
       pad->buttonStatus = (pad->buttonStatus>>8)|(pad->buttonStatus<<8);
    }

    return 0;
}

long PADkeypressed(void) {
    return 0;
}

long PADconfigure(void) {
    return 0;
}


void PADabout(void) {
}

long PADtest(void) {
    return 0;
}
